//
//  TSMenuHelperView.m
//  MenuHelper
//
//  Created by Robin Senior on 2013-05-13.
//  Copyright (c) 2013 theScore. All rights reserved.
//

#import "TSMenuHelperView.h"
#import <QuartzCore/QuartzCore.h>

@interface TSMenuHelperView ()
{
	CAShapeLayer *_borderLayer;
	CAShapeLayer *_touchLayer;
	CAShapeLayer *_circleLayer;
	CAShapeLayer *_arcLayer;
}
- (void)TS_commonMenuHelperViewInit;
@end

@implementation TSMenuHelperView

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self)
	{
		[self TS_commonMenuHelperViewInit];
    }
    return self;
}

-(id)initWithCoder:(NSCoder *)aDecoder
{
	self = [super initWithCoder:aDecoder];
	if(self)
	{
		[self TS_commonMenuHelperViewInit];
	}
	return self;
}

- (void)TS_commonMenuHelperViewInit
{
	[self setBackgroundColor:[UIColor lightGrayColor]];
	
	_borderLayer = [[CAShapeLayer alloc] init];
	[_borderLayer setLineWidth:1];
	[_borderLayer setStrokeColor:[[UIColor greenColor] CGColor]];
	[_borderLayer setFillColor:[[UIColor clearColor] CGColor]];
	
	[[self layer] addSublayer:_borderLayer];
	
	_circleLayer = [[CAShapeLayer alloc] init];
	[_circleLayer setLineWidth:1];
	[_circleLayer setStrokeColor:[[UIColor redColor] CGColor]];
	[_circleLayer setFillColor:[[UIColor clearColor] CGColor]];
	
	[[self layer] addSublayer:_circleLayer];
	
	_arcLayer = [[CAShapeLayer alloc] init];
	[_arcLayer setLineWidth:1];
	[_arcLayer setStrokeColor:[[UIColor blueColor] CGColor]];
	[_arcLayer setFillColor:[[UIColor colorWithRed:0 green:0 blue:1 alpha:.2] CGColor]];
	
	[[self layer] addSublayer:_arcLayer];
	
	_touchLayer = [[CAShapeLayer alloc] init];
	[_touchLayer setLineWidth:0];
	[_touchLayer setStrokeColor:[[UIColor clearColor] CGColor]];
	[_touchLayer setFillColor:[[UIColor redColor] CGColor]];
	
	[[self layer] addSublayer:_touchLayer];
}

- (void)reloadData
{
	CGRect borderRect = [[self delegate] drawableRectForRect:[self bounds]];
	CGPathRef path = CGPathCreateWithRect(borderRect, NULL);
	
	[_borderLayer setPath:path];
	
	CGPathRelease(path);
}

-(void)layoutSublayersOfLayer:(CALayer *)layer
{
	[super layoutSublayersOfLayer:layer];
	
	[self reloadData];
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [touches anyObject];
	
	[self TS_updateWithPoint:[touch locationInView:self]];
}

-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [touches anyObject];
	
	[self TS_updateWithPoint:[touch locationInView:self]];
}

- (void)TS_updateWithPoint:(CGPoint)touchPoint
{
	CGPoint point = [[self delegate] drawablePointForPoint:touchPoint inRect:[self bounds]];
	CGFloat radius = 5;
	
	CGPathRef touchPath = CGPathCreateWithEllipseInRect(CGRectMake(point.x - radius, point.y - radius, radius * 2, radius * 2), NULL);
	[_touchLayer setPath:touchPath];
	CGPathRelease(touchPath);
	
	radius = [[self delegate] radius];
	CGPathRef circlePath = CGPathCreateWithEllipseInRect(CGRectMake(point.x - radius, point.y - radius, radius * 2, radius * 2), NULL);
	[_circleLayer setPath:circlePath];
	CGPathRelease(circlePath);
	
	CGPathRef arcPath = [[self delegate] newDrawableArcForPoint:point inRect:[self bounds]];
	[_arcLayer setPath:arcPath];
	CGPathRelease(arcPath);
}

@end
