//
//  TSViewController.m
//  MenuHelper
//
//  Created by Robin Senior on 2013-05-13.
//  Copyright (c) 2013 theScore. All rights reserved.
//

#import "TSViewController.h"
#import <math.h>

@interface TSViewController ()

@end

@implementation TSViewController

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

-(CGRect)drawableRectForRect:(CGRect)rect
{
	return CGRectInset(rect, 20, 20);
}

- (CGPoint)drawablePointForPoint:(CGPoint)point inRect:(CGRect)rect
{
	CGRect drawableRect = [self drawableRectForRect:rect];
	
	point.x = MIN(point.x, CGRectGetMaxX(drawableRect));
	point.x = MAX(point.x, CGRectGetMinX(drawableRect));
	
	point.y = MIN(point.y, CGRectGetMaxY(drawableRect));
	point.y = MAX(point.y, CGRectGetMinY(drawableRect));
	
	return point;
}

-(CGPathRef)newDrawableArcForPoint:(CGPoint)point inRect:(CGRect)rect
{
	CGPathRef arc;
	
	CGRect drawableRect = [self drawableRectForRect:rect];
	CGPoint drawablePoint = [self drawablePointForPoint:point inRect:rect];
	CGFloat radius = [self radius];
	
	// if no intersection
	CGRect hitRect = CGRectMake(drawablePoint.x-radius, drawablePoint.y-radius, radius*2, radius*2);
	BOOL contains = CGRectContainsRect(drawableRect, hitRect);
	
	if (contains)
	{
		arc = CGPathCreateWithEllipseInRect(hitRect, NULL);
		return arc;
	}
	else
	{
		CGFloat minX = CGRectGetMinX(drawableRect);
		CGFloat maxX = CGRectGetMaxX(drawableRect);
		CGFloat minY = CGRectGetMinY(drawableRect);
		CGFloat maxY = CGRectGetMaxY(drawableRect);

		CGFloat x = drawablePoint.x;
		CGFloat y = drawablePoint.y;
		
		CGRect testRect = CGRectMake(minX, minY, radius * 2, radius * 2);
		CGPoint testPoint = CGPointMake(x+radius, y+radius);

		// top left
		if (CGRectContainsPoint(testRect, testPoint))
		{
			CGFloat dy = y - minY;
			CGFloat cosTheta = dy/radius;
			CGFloat endAngle = -(M_PI_2 - acosf(cosTheta));
			
			CGFloat dx = x - minX;
			CGFloat cosGamma = dx/radius;
			CGFloat startAngle = M_PI - acosf(cosGamma);
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// top right
		testRect = CGRectMake(maxX - (2*radius), minY, radius*2, radius*2);
		testPoint = CGPointMake(x-radius, y+radius);
		
		if (CGRectContainsPoint(testRect, testPoint))
		{
			CGFloat dy = y - minY;
			CGFloat cosTheta = dy/radius;
			CGFloat startAngle = -(acosf(cosTheta))-M_PI_2;
			
			CGFloat dx = maxX - x;
			CGFloat cosGamma = dx/radius;
			CGFloat endAngle = acosf(cosGamma);
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// bottom left
		testRect = CGRectMake(minX, maxY-(radius*2), radius*2, radius*2);
		testPoint = CGPointMake(x+radius, y-radius);
		
		if (CGRectContainsPoint(testRect, testPoint))
		{
			CGFloat dy = maxY - y;
			CGFloat cosTheta = dy/radius;
			CGFloat startAngle = M_PI_2 - acosf(cosTheta);
			
			CGFloat dx = x - minX;
			CGFloat cosGamma = dx/radius;
			CGFloat endAngle = M_PI + acosf(cosGamma);
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// bottom right
		testRect = CGRectMake(maxX-(radius*2), maxY-(radius*2), radius*2, radius*2);
		testPoint = CGPointMake(x-radius, y-radius);
		
		if (CGRectContainsPoint(testRect, testPoint))
		{
			CGFloat dy = maxY - y;
			CGFloat cosTheta = dy/radius;
			CGFloat endAngle = M_PI_2 + acosf(cosTheta);
			
			CGFloat dx = maxX - x;
			CGFloat cosGamma = dx/radius;
			CGFloat startAngle = -acosf(cosGamma);
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// left
		if (CGRectGetMinX(hitRect) < minX)
		{
			CGFloat dx = x - minX;
			CGFloat cosTheta = dx/radius;
			CGFloat startAngle = (M_PI) - acosf(cosTheta);
			CGFloat endAngle = -startAngle;
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// right
		if (CGRectGetMaxX(hitRect) > maxX)
		{
			CGFloat dx = maxX - x;
			CGFloat cosTheta = dx/radius;
			CGFloat endAngle = acosf(cosTheta);
			CGFloat startAngle = -endAngle;
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// top
		if (CGRectGetMinY(hitRect) < minY)
		{
			CGFloat dy = y - minY;
			CGFloat sinTheta = dy/radius;
			CGFloat endAngle = -asinf(sinTheta);
			CGFloat startAngle = M_PI - endAngle;
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		// bottom
		if (CGRectGetMaxY(hitRect) > maxY)
		{
			CGFloat dy = maxY - y;
			CGFloat sinTheta = dy/radius;
			CGFloat startAngle = asinf(sinTheta);
			CGFloat endAngle = M_PI - startAngle;
			
			return [self newArcPathAtPoint:drawablePoint betweenStartAngle:startAngle endAngle:endAngle];
		}
		
		return nil;
	}
}

- (CGPathRef)newArcPathAtPoint:(CGPoint)center betweenStartAngle:(CGFloat)start endAngle:(CGFloat)end
{	
	CGFloat radius = [self radius];

	CGFloat startX = center.x + (cos(start) * radius);
	CGFloat startY = center.y + (sin(start) * radius);
	
	CGMutablePathRef path = CGPathCreateMutable();

	CGPathMoveToPoint(path, NULL, center.x, center.y);
	CGPathAddLineToPoint(path, NULL, startX, startY);
	
	// if you want to display a certain number of menu items, instead of using CGPathAddArc,
	// simply iterate between the start and end angles to create a path with the required number of points.
	// these points can be used to lay out your menu items
	
	CGPathAddArc(path, NULL, center.x, center.y, radius, start, end, YES);
	
	CGPathCloseSubpath(path);
	
	return path;
}

-(CGFloat)radius
{
	return 50;
}

-(NSUInteger)numberOfMenuItems
{
	return 20;
}

- (CGAffineTransform)rotateAngle:(CGFloat)angle aroundCenter:(CGPoint)center
{
	CGAffineTransform translateTransform = CGAffineTransformMakeTranslation(center.x, center.y);
	CGAffineTransform rotationTransform = CGAffineTransformMakeRotation(angle);
	
	CGAffineTransform customRotation = CGAffineTransformConcat(CGAffineTransformConcat( CGAffineTransformInvert(translateTransform), rotationTransform), translateTransform);
	
	return customRotation;
}

@end
